/*
 * p000_invertor_fun_cooler.c
 *
 * Created: 06.07.2013 15:47:25
 *  Author: axillent
 */

#define INSOURCE

// VCC = 5V
// PB0 - PWM output
// PB1 - voltage analog input
// PB2 - temperature analog input
#define TEMPERATURE_ADC	2
#define VOLTAGE_ADC		1

#define MAX_VOLTAGE_ADC	204		// maximum 12v on fan (using AREF=5V and voltage devider 1/3)
#define MIN_VOLTAGE_ADC	98		// minimum voltage on fan
#define MIN_TEMPERATURE	34
#define MAX_TEMPERATURE	46
#define OFF_TEMPERATURE	32
#define PWM_TOP			127

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <util/delay.h>

volatile uint8_t	voltage;
volatile uint8_t	target_voltage;

ISR(ADC_vect) {
	switch(ADMUX) {
		case TEMPERATURE_ADC:
		{
			// formula - (adcl * 50 /4 * 10 + 32) / 64 - 273
			uint16_t temperature = ADCL;
			temperature *= 50;
			temperature /= 4;
			temperature *= 10;
			temperature += 32;
			temperature /= 64;
			if(temperature > 273) temperature -= 273;
			else temperature = 0;

			// getting 32 range
			if(temperature <= OFF_TEMPERATURE) {
				target_voltage = 0;
			} else {
				temperature = (temperature < MIN_TEMPERATURE)?0:(temperature > MAX_TEMPERATURE)?(MAX_TEMPERATURE - MIN_TEMPERATURE):temperature-MIN_TEMPERATURE;
				temperature *= (MAX_VOLTAGE_ADC - MIN_VOLTAGE_ADC);
				temperature /= (MAX_TEMPERATURE - MIN_TEMPERATURE);
				target_voltage = MIN_VOLTAGE_ADC + temperature;
			}

			ADMUX = VOLTAGE_ADC;
			break;
		}
		case VOLTAGE_ADC:
			voltage = ADCL;
			ADMUX = TEMPERATURE_ADC;
			break;
	}
	ADCSRA |= (1 << ADSC);
}

int main(void)
{
	// configure core, run at 8mhz
	CCP = 0xD8;
	CLKPSR = 0;

	// ports
	DDRB |= (1 << PB0);

	//configure PWM
	ICR0 = PWM_TOP;
	TCCR0A = (1 << WGM01) | (0 << WGM00) | (1 << COM0A1) | (0 << COM0A0);
	TCCR0B = (1 << CS00) | (1 << WGM03) | (1 << WGM02);

	// configure ADC
	// start from temperature
	ADMUX = TEMPERATURE_ADC;
	// enable, /32 prescaler, enable interrupts, trigger source
	ADCSRA = (1 << ADEN) | (1 << ADIE) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);
	// free running mode
	ADCSRB = 0;

	// enable interrupts
	sei();
	// start first conversion
	ADCSRA |= (1 << ADSC);

	wdt_enable(WDTO_1S);

    while(1)
    {
		if(target_voltage) {
			int8_t diff = target_voltage - voltage;
			TCCR0A |= ((1 << COM0A1) | (0 << COM0A0));
			if(diff > 0) {
				if(OCR0A < PWM_TOP) OCR0A++;
			} else if(diff < 0) {
				if(OCR0A) OCR0A--;
			}
		} else {
			TCCR0A &= ~((1 << COM0A1) | (0 << COM0A0));
			PORTB &= ~(1 << PINB0);
		}
		wdt_reset();
		_delay_ms(50);
    }
}